@@ -6,9 +6,10 @@ use std::process::{Child, Command, Stdio};
 use std::sync::Arc;
 use std::sync::atomic::{AtomicBool, AtomicUsize, Ordering};
 use std::thread;
+use std::time::{Duration, Instant};
 
 use regex::Regex;
-use tokio::sync::mpsc::{UnboundedReceiver, UnboundedSender, unbounded_channel};
+use tokio::sync::mpsc::{UnboundedSender, unbounded_channel};
 
 use crate::field::FieldRange;
 use crate::helper::item::DefaultSkimItem;
@@ -17,6 +18,8 @@ use crate::{SkimItem, SkimItemReceiver, SkimItemSender, SkimOptions};
 
 const DELIMITER_STR: &str = r"[\t\n ]+";
 const READ_BUFFER_SIZE: usize = 1024;
+const ITEMS_BUFFER_SIZE: usize = 128;
+const SEND_TIMEOUT_MS: u64 = 100; // Send items if we haven't sent anything in 100ms
 
 pub enum CollectorInput {
     Pipe(Box<dyn BufRead + Send>),
@@ -191,50 +194,90 @@ impl SkimItemReader {
         }
     }
 
-    /// helper: convert bufread into SkimItemReceiver
-    fn raw_bufread(&self, mut source: impl BufRead + Send + 'static) -> SkimItemReceiver {
-        let (tx_item, rx_item): (SkimItemSender, SkimItemReceiver) = unbounded_channel();
-        let line_ending = self.option.line_ending;
-        let use_ansi = self.option.use_ansi_color;
-        let delimiter = self.option.delimiter.clone();
-        thread::spawn(move || {
-            let mut buffer = Vec::with_capacity(1024);
-            let mut idx = 0;
-            loop {
-                buffer.clear();
-                // start reading
-                match source.read_until(line_ending, &mut buffer) {
-                    Ok(n) => {
-                        if n == 0 {
-                            break;
-                        }
+    /// Helper function that contains the common logic for reading lines from a BufRead source
+    /// and converting them into SkimItems.
+    fn read_lines_into_items(
+        mut source: impl BufRead + Send + 'static,
+        tx_item: SkimItemSender,
+        option: Arc<SkimItemReaderOption>,
+        transform_fields: Vec<FieldRange>,
+        matching_fields: Vec<FieldRange>,
+    ) {
+        let mut buffer = Vec::with_capacity(option.buf_size);
+        let mut line_idx = 0;
+        let mut items_to_send = Vec::with_capacity(ITEMS_BUFFER_SIZE);
+        let mut last_send_time = Instant::now();
+        let send_timeout = Duration::from_millis(SEND_TIMEOUT_MS);
 
-                        if buffer.ends_with(b"\r\n") {
-                            buffer.pop();
-                            buffer.pop();
-                        } else if buffer.ends_with(b"\n") || buffer.ends_with(b"\0") {
-                            buffer.pop();
-                        }
+        loop {
+            buffer.clear();
 
-                        let string = String::from_utf8_lossy(&buffer);
-                        //let result = tx_item.send(Arc::new(string.into_owned()));
-                        let result = tx_item.send(Arc::new(DefaultSkimItem::new(
-                            string.to_string(),
-                            use_ansi,
-                            &[],
-                            &[],
-                            &delimiter,
-                            idx,
-                        )));
-                        if result.is_err() {
-                            break;
-                        }
-                        idx += 1;
+            // start reading
+            match source.read_until(option.line_ending, &mut buffer) {
+                Ok(0) => break,
+                Ok(_) => {
+                    // Strip line endings
+                    if buffer.ends_with(b"\r\n") {
+                        buffer.pop();
+                        buffer.pop();
+                    } else if buffer.ends_with(&[option.line_ending]) {
+                        buffer.pop();
+                    }
+
+                    let line = String::from_utf8_lossy(&buffer).to_string();
+
+                    trace!("got item {} with index {}", line.clone(), line_idx);
+
+                    let raw_item = DefaultSkimItem::new(
+                        line,
+                        option.use_ansi_color,
+                        &transform_fields,
+                        &matching_fields,
+                        &option.delimiter,
+                        line_idx,
+                    );
+                    items_to_send.push(Arc::new(raw_item) as Arc<dyn SkimItem>);
+
+                    line_idx += 1;
+                }
+                Err(err) => {
+                    trace!("Got {err:?} when reading, skipping");
+                } // String not UTF8 or other error, skip.
+            }
+
+            // Send batched items if buffer is full OR timeout has elapsed
+            let should_send = items_to_send.len() == ITEMS_BUFFER_SIZE
+                || (!items_to_send.is_empty() && last_send_time.elapsed() >= send_timeout);
+
+            if should_send {
+                let batch = std::mem::replace(&mut items_to_send, Vec::with_capacity(ITEMS_BUFFER_SIZE));
+                match tx_item.send(batch) {
+                    Ok(_) => {
+                        last_send_time = Instant::now();
+                    }
+                    Err(e) => {
+                        warn!("Failed to send items: {e:?}");
+                        break;
                     }
-                    Err(_err) => {} // String not UTF8 or other error, skip.
                 }
             }
+        }
+
+        // Send remaining items
+        if !items_to_send.is_empty() {
+            let _ = tx_item.send(items_to_send);
+        }
+    }
+
+    /// helper: convert bufread into SkimItemReceiver
+    fn raw_bufread(&self, source: impl BufRead + Send + 'static) -> SkimItemReceiver {
+        let (tx_item, rx_item): (SkimItemSender, SkimItemReceiver) = unbounded_channel();
+        let option = self.option.clone();
+
+        thread::spawn(move || {
+            Self::read_lines_into_items(source, tx_item, option, vec![], vec![]);
         });
+
         rx_item
     }
 
@@ -244,15 +287,15 @@ impl SkimItemReader {
         &self,
         components_to_stop: Arc<AtomicUsize>,
         input: CollectorInput,
-    ) -> (UnboundedReceiver<Arc<dyn SkimItem>>, UnboundedSender<i32>) {
+    ) -> (SkimItemReceiver, UnboundedSender<i32>) {
         let send_error = self.option.show_error;
-        let (command, mut source) = match input {
+        let (command, source) = match input {
             CollectorInput::Pipe(pipe) => (None, pipe),
             CollectorInput::Command(cmd) => get_command_output(&cmd, send_error).expect("command not found"),
         };
 
         let (tx_interrupt, mut rx_interrupt) = unbounded_channel();
-        let (tx_item, rx_item): (SkimItemSender, SkimItemReceiver) = unbounded_channel::<Arc<dyn SkimItem>>();
+        let (tx_item, rx_item): (SkimItemSender, SkimItemReceiver) = unbounded_channel();
 
         let started = Arc::new(AtomicBool::new(false));
         let started_clone = started.clone();
@@ -278,9 +321,21 @@ impl SkimItemReader {
                     if has_error {
                         trace!("collector: sending error");
                         let output = child.wait_with_output().expect("could not retrieve error message");
-                        for line in String::from_utf8_lossy(&output.stderr).lines() {
-                            let _ = tx_item_clone.send(Arc::new(line.to_string()));
-                        }
+                        let error_text = String::from_utf8_lossy(&output.stderr).to_string();
+                        let error_items: Vec<Arc<dyn SkimItem>> = error_text
+                            .lines()
+                            .map(|line| {
+                                Arc::new(DefaultSkimItem::new(
+                                    line.to_string(),
+                                    false,
+                                    &[],
+                                    &[],
+                                    &Regex::new(DELIMITER_STR).unwrap(),
+                                    0,
+                                )) as Arc<dyn SkimItem>
+                            })
+                            .collect();
+                        let _ = tx_item_clone.send(error_items);
                     }
                 }
             }
@@ -297,54 +352,15 @@ impl SkimItemReader {
         let started_clone = started.clone();
         let tx_interrupt_clone = tx_interrupt.clone();
         let option = self.option.clone();
+        let transform_fields = option.transform_fields.clone();
+        let matching_fields = option.matching_fields.clone();
+
         thread::spawn(move || {
             debug!("collector: command collector start");
             components_to_stop.fetch_add(1, Ordering::SeqCst);
             started_clone.store(true, Ordering::SeqCst); // notify parent that it is started
 
-            let mut buffer = Vec::with_capacity(option.buf_size);
-            let mut line_idx = 0;
-            loop {
-                buffer.clear();
-
-                // start reading
-                match source.read_until(option.line_ending, &mut buffer) {
-                    Ok(0) => break,
-                    Ok(_) => {
-                        if buffer.ends_with(b"\r\n") {
-                            buffer.pop();
-                            buffer.pop();
-                        } else if buffer.ends_with(&[option.line_ending]) {
-                            buffer.pop();
-                        }
-
-                        let line = String::from_utf8_lossy(&buffer).to_string();
-
-                        trace!("got item {} with index {} from command", line.clone(), line_idx);
-
-                        let raw_item = DefaultSkimItem::new(
-                            line,
-                            option.use_ansi_color,
-                            &option.transform_fields,
-                            &option.matching_fields,
-                            &option.delimiter,
-                            line_idx,
-                        );
-
-                        match tx_item.send(Arc::new(raw_item)) {
-                            Ok(_) => {}
-                            Err(_) => {
-                                debug!("collector: failed to send item, quit");
-                                break;
-                            }
-                        }
-                        line_idx += 1;
-                    }
-                    Err(err) => {
-                        trace!("Got {err:?} when reading from command collector, skipping");
-                    } // String not UTF8 or other error, skip.
-                }
-            }
+            Self::read_lines_into_items(source, tx_item, option, transform_fields, matching_fields);
 
             let _ = tx_interrupt_clone.send(1); // ensure the waiting thread will exit
             components_to_stop.fetch_sub(1, Ordering::SeqCst);
