@@ -64,7 +64,7 @@ pub mod field;
 pub mod fuzzy_matcher;
 pub mod helper;
 pub mod item;
-mod matcher;
+pub mod matcher;
 pub mod options;
 mod output;
 pub mod prelude;
@@ -328,9 +328,9 @@ pub trait Selector {
 
 //------------------------------------------------------------------------------
 /// Sender for streaming items to skim
-pub type SkimItemSender = UnboundedSender<Arc<dyn SkimItem>>;
+pub type SkimItemSender = UnboundedSender<Vec<Arc<dyn SkimItem>>>;
 /// Receiver for streaming items to skim
-pub type SkimItemReceiver = UnboundedReceiver<Arc<dyn SkimItem>>;
+pub type SkimItemReceiver = UnboundedReceiver<Vec<Arc<dyn SkimItem>>>;
 
 /// Main entry point for running skim
 pub struct Skim {}
@@ -399,7 +399,6 @@ impl Skim {
 
             let item_pool = app.item_pool.clone();
             tokio::spawn(async move {
-                const BATCH: usize = 4096; // Smaller batches for more responsive updates
                 loop {
                     if reader_interrupt_rx
                         .try_recv()
@@ -408,13 +407,16 @@ impl Skim {
                         debug!("stopping reader receiver thread");
                         break;
                     }
-                    let mut buf = Vec::with_capacity(BATCH);
                     trace!("getting items");
-                    if item_rx.recv_many(&mut buf, BATCH).await > 0 {
-                        item_pool.append(buf);
-                        trace!("Got new items, len {}", item_pool.len());
-                    } else {
-                        reader_done_clone.store(true, std::sync::atomic::Ordering::Relaxed);
+                    match item_rx.recv().await {
+                        Some(batch) => {
+                            item_pool.append(batch);
+                            trace!("Got new items, len {}", item_pool.len());
+                        }
+                        None => {
+                            reader_done_clone.store(true, std::sync::atomic::Ordering::Relaxed);
+                            break;
+                        }
                     }
                 }
             });
@@ -431,28 +433,11 @@ impl Skim {
                 0
             };
 
-            let listener = if let Some(socket_name) = &listen_socket {
-                debug!("starting listener on socket at {socket_name}");
-                Some(
-                    interprocess::local_socket::ListenerOptions::new()
-                        .name(
-                            interprocess::local_socket::ToNsName::to_ns_name::<
-                                interprocess::local_socket::GenericNamespaced,
-                            >(socket_name.as_str())
-                            .unwrap_or_else(|e| {
-                                panic!("Failed to build full name for IPC listener from {socket_name}: {e}")
-                            }),
-                        )
-                        .create_tokio()
-                        .unwrap_or_else(|e| panic!("Failed to create tokio IPC listener at {socket_name}: {e}")),
-                )
-            } else {
-                None
-            };
-
             let mut should_enter = true;
             if min_items_before_enter > 0 {
-                while app.matcher_control.get_num_matched() < min_items_before_enter && app.item_pool.num_not_taken() > 0 && !app.matcher_control.stopped() {
+                while app.matcher_control.get_num_matched() < min_items_before_enter
+                    && !app.matcher_control.stopped()
+                    && !reader_done.load(std::sync::atomic::Ordering::Relaxed) {
                     let curr = app.matcher_control.get_num_matched();
                     tokio::time::sleep(Duration::from_millis(10)).await;
                     trace!(
@@ -468,7 +453,7 @@ impl Skim {
                     app.restart_matcher(false);
                 };
                 trace!("checking for matched item count before entering: {}/{min_items_before_enter}", app.matcher_control.get_num_matched());
-                if app.matcher_control.stopped() && app.matcher_control.get_num_matched() == min_items_before_enter - 1
+                if app.matcher_control.get_num_matched() == min_items_before_enter - 1
                 {
                     app.item_list.items = app.item_list.processed_items.lock().take().unwrap_or_default().items;
                     debug!("early exit, result: {:?}", app.results());
@@ -478,77 +463,97 @@ impl Skim {
             }
 
             if should_enter {
-            let backend = CrosstermBackend::new(std::io::BufWriter::new(std::io::stderr()));
-            let mut tui = tui::Tui::new_with_height(backend, height)?;
-            let event_tx_clone = tui.event_tx.clone();
-            tui.enter()?;
-            loop {
-                select! {
-                    event = tui.next() => {
-                        let evt = event.ok_or_eyre("Could not acquire next event")?;
+                let listener = if let Some(socket_name) = &listen_socket {
+                    debug!("starting listener on socket at {socket_name}");
+                    Some(
+                        interprocess::local_socket::ListenerOptions::new()
+                            .name(
+                                interprocess::local_socket::ToNsName::to_ns_name::<
+                                    interprocess::local_socket::GenericNamespaced,
+                                >(socket_name.as_str())
+                                .unwrap_or_else(|e| {
+                                    panic!("Failed to build full name for IPC listener from {socket_name}: {e}")
+                                }),
+                            )
+                            .create_tokio()
+                            .unwrap_or_else(|e| panic!("Failed to create tokio IPC listener at {socket_name}: {e}")),
+                    )
+                } else {
+                    None
+                };
+                let backend = CrosstermBackend::new(std::io::BufWriter::new(std::io::stderr()));
+                let mut tui = tui::Tui::new_with_height(backend, height)?;
+                let event_tx_clone = tui.event_tx.clone();
+                tui.enter()?;
+                loop {
+                    select! {
+                        event = tui.next() => {
+                            let evt = event.ok_or_eyre("Could not acquire next event")?;
 
-                        // Handle reload event
-                        if let Event::Reload(new_cmd) = &evt {
-                            debug!("reloading with cmd {new_cmd}");
-                            // Kill the current reader
-                            reader_control.kill();
-                            // Clear items
-                            app.item_pool.clear();
-                            // Clear displayed items unless no_clear_if_empty is set
-                            // (in which case the item_list will handle keeping stale items)
-                            if !app.options.no_clear_if_empty {
-                                app.item_list.clear();
-                            }
-                            app.restart_matcher(true);
-                            // Start a new reader with the new command (no source, using cmd)
-                            reader_control = reader.run(item_tx.clone(), new_cmd);
-                            reader_done.store(false, std::sync::atomic::Ordering::Relaxed);
-                        }
-                        if let Event::Key(k) = &evt {
-                          final_key = k.to_owned();
-                        } else {
-                          final_event = evt.to_owned();
-                        }
-
-                        // Check reader status and update
-                        if !reader_control.is_done() {
-                          app.reader_timer = Instant::now();
-                        } else if ! reader_done.load(std::sync::atomic::Ordering::Relaxed) {
-                            reader_done.store(true, std::sync::atomic::Ordering::Relaxed);
-                            app.restart_matcher(true);
-                        }
-                        app.handle_event(&mut tui, &evt)?;
-                    }
-                    _ = matcher_interval.tick() => {
-                      app.restart_matcher(false);
-                    }
-                    Ok(stream) = async {
-                        match &listener {
-                            Some(l) => interprocess::local_socket::traits::tokio::Listener::accept(l).await,
-                            None => std::future::pending().await,
-                        }
-                    } => {
-                        let event_tx_clone_ipc = event_tx_clone.clone();
-                        tokio::spawn(async move {
-                            use tokio::io::AsyncBufReadExt;
-                            let reader = tokio::io::BufReader::new(stream);
-                            let mut lines = reader.lines();
-                            while let Ok(Some(line)) = lines.next_line().await {
-                                debug!("listener: got {line}");
-                                if let Ok(act) = ron::from_str::<Action>(&line) {
-                                    debug!("listener: parsed into action {act:?}");
-                                    _ = event_tx_clone_ipc.send(Event::Action(act));
-                                    _ = event_tx_clone_ipc.send(Event::Render);
+                            // Handle reload event
+                            if let Event::Reload(new_cmd) = &evt {
+                                debug!("reloading with cmd {new_cmd}");
+                                // Kill the current reader
+                                reader_control.kill();
+                                // Clear items
+                                app.item_pool.clear();
+                                // Clear displayed items unless no_clear_if_empty is set
+                                // (in which case the item_list will handle keeping stale items)
+                                if !app.options.no_clear_if_empty {
+                                    app.item_list.clear();
                                 }
+                                app.restart_matcher(true);
+                                // Start a new reader with the new command (no source, using cmd)
+                                reader_control = reader.run(item_tx.clone(), new_cmd);
+                                reader_done.store(false, std::sync::atomic::Ordering::Relaxed);
                             }
-                        });
+                            if let Event::Key(k) = &evt {
+                              final_key = k.to_owned();
+                            } else {
+                              final_event = evt.to_owned();
+                            }
+
+                            // Check reader status and update
+                            if !reader_control.is_done() {
+                              app.reader_timer = Instant::now();
+                            } else if ! reader_done.load(std::sync::atomic::Ordering::Relaxed) {
+                                reader_done.store(true, std::sync::atomic::Ordering::Relaxed);
+                                app.restart_matcher(false);
+                            }
+                            app.handle_event(&mut tui, &evt)?;
+                        }
+                        _ = matcher_interval.tick() => {
+                          app.restart_matcher(false);
+                        }
+                        Ok(stream) = async {
+                            match &listener {
+                                Some(l) => interprocess::local_socket::traits::tokio::Listener::accept(l).await,
+                                None => std::future::pending().await,
+                            }
+                        } => {
+                            let event_tx_clone_ipc = event_tx_clone.clone();
+                            tokio::spawn(async move {
+                                use tokio::io::AsyncBufReadExt;
+                                let reader = tokio::io::BufReader::new(stream);
+                                let mut lines = reader.lines();
+                                while let Ok(Some(line)) = lines.next_line().await {
+                                    debug!("listener: got {line}");
+                                    if let Ok(act) = ron::from_str::<Action>(&line) {
+                                        debug!("listener: parsed into action {act:?}");
+                                        _ = event_tx_clone_ipc.send(Event::Action(act));
+                                        _ = event_tx_clone_ipc.send(Event::Render);
+                                    }
+                                }
+                            });
+                        }
+                    }
+
+                    if app.should_quit {
+                        app.preview.kill();
+                        app.matcher_control.kill();
+                        break;
                     }
                 }
-
-                if app.should_quit {
-                    break;
-                }
-            }
             }
             reader_control.kill();
             let _ = reader_interrupt_tx.send(true);
@@ -558,7 +563,7 @@ impl Skim {
         // Extract final_key and is_abort from final_event
         let is_abort = !matches!(&final_event, Event::Action(Action::Accept(_)));
 
-        Ok(SkimOutput {
+        let output = SkimOutput {
             cmd: if app.options.interactive {
                 // In interactive mode, cmd is what the user typed
                 app.input.to_string()
@@ -574,7 +579,12 @@ impl Skim {
             query: app.input.to_string(),
             is_abort,
             selected_items: app.results(),
-            header: app.header.header,
-        })
+            header: app.header.header.clone(),
+        };
+        // Explicitely drop app to make sure we stop all components
+        drop(app);
+        debug!("output: {output:?}");
+
+        Ok(output)
     }
 }
