@@ -164,6 +164,12 @@ pub struct SkimOptions {
     )]
     pub case: CaseMatching,
 
+    /// Normalize unicode characters
+    ///
+    /// When set, normalize accents and other unicode diacritics/others
+    #[cfg_attr(feature = "cli", arg(long, help_heading = "Search"))]
+    pub normalize: bool,
+
     /// Enable split matching and set delimiter
     ///
     /// Split matching runs the matcher in splits: `foo:bar` will match all items matching `foo`, then
@@ -280,7 +286,7 @@ pub struct SkimOptions {
     /// Line  will  start  with  the  start of the matched pattern. Effective only when the query
     /// string is empty. Was designed to skip showing starts of paths of rg/grep results.
     ///
-    /// e.g. sk -i -c "rg {} --color=always" --skip-to-pattern '[^/]*:' --ansi
+    /// e.g. sk -i -c "rg {q} --color=always" --skip-to-pattern '[^/]*:' --ansi
     #[cfg_attr(feature = "cli", arg(long, help_heading = "Interface", verbatim_doc_comment))]
     pub skip_to_pattern: Option<String>,
 
@@ -779,6 +785,7 @@ impl Default for SkimOptions {
             regex: Default::default(),
             algorithm: Default::default(),
             case: Default::default(),
+            normalize: false,
             bind: Default::default(),
             multi: Default::default(),
             no_multi: Default::default(),
